require 'tatami'
require 'awesome_print'
require 'minitest/autorun'
module Tatami
  describe Couch do
    let(:couch){ Couch.new 'cars' }
    let :couch_one do
      couch.blank!.doc_new :frank, name: 'Frank'
      couch
    end

    it "setup" do
      couch.version.must_equal '1.3.1'
      couch.db_info[:db_name].must_equal 'cars'
    end

    it "#load_design" do
      couch.with_fixtures! a: 1
    end

    describe Couch::View do

      it '#view' do
        couch_one.view('v1').create('(doc) -> emit "c", 0').keys.must_equal ['c']
        couch_one.view('v2').create('
  (doc) ->
    emit "d", 1
    emit "e", 2','
  (keys, values, rereduce) ->
    sum(values)').reduced.must_equal 3
      end

      #View options
      it 'string datatypes' do
        v = couch_one.view('v3').create '(doc) ->
   emit "a", 1
   emit "b", 2
   emit "c", 3'
        v.keys.must_equal %w[a b c]
        v.keys(startkey: 'b'     ).must_equal %w[  b c]
        v.keys(endkey:   'b'     ).must_equal %w[a b  ]
        v.keys(key:      'b'     ).must_equal %w[  b  ]
        v.keys(limit:     2      ).must_equal %w[a b  ]
        v.keys(skip:      1      ).must_equal %w[  b c]
        v.keys(descending: true  ).must_equal %w[c b a]
        v.keys(keys:      %w[a c]).must_equal %w[a   c]
      end

      it "sorts different datatypes" do
        couch_one.view('a').create('(doc) ->
 emit null, 0
 emit 0, 0
 emit "", 0
 emit [], 0
 emit [1], 0
 emit [1,1], 0
 emit {}, 0').keys.must_equal [nil,0,'',[],[1],[1,1],{}]
      end

      it 'Map Reduce' do
        couch.with_fixtures! %w[make model year],
                                    ['Audi','A3',2000],
                                    ['Audi','A4',2009],
                                    ['VW','Golf',2008],
                                    ['VW','Golf',2009],
                                    ['VW','Polo',2010]

        primitive = couch.view('primitive').create('
(doc) -> emit doc.make , 1','
(keys, values, rereduce) -> sum(values)')

        primitive.data.get(:rows,0)       .must_equal "key" => nil, "value" => 5
        primitive.data(group: true)[:rows].must_equal [{"key"=>"Audi", "value"=>2},
                                                         {"key"=>"VW", "value"=>3}]

        primitive.reduce( group: true).must_equal 'Audi' => 2, 'VW' => 3
        primitive.reduced.must_equal 5

        array = couch.view('array').create('
(doc) -> emit [doc.make,doc.model] , 1','
(keys, values, rereduce) -> sum(values)')
        array.reduced.must_equal 5
        array.reduce(     group_level: 1).must_equal ['Audi'] => 2, ['VW'] => 3
        array.reduced(    startkey: ['Audi'], endkey: ['Audi',{}]).must_equal 2
        array.reduce(     group_level: 1,startkey: ['Audi'], endkey: ['Audi',{}]).must_equal ['Audi'] => 2
        array.reduced(    group_level: 1,startkey: ['Audi'], endkey: ['Audi',{}]).must_equal 2
        array.reduced_value('Audi').must_equal 2
        array.reduced_value('VW','Golf').must_equal 2
      end

      describe "list action" do

        let(:v1){couch.view(:v1).create('(doc) -> emit doc.model, 1')}

        it 'show list simple' do
          couch.design_new :list, :zoom, '() -> "zoom!"'
          out, err = capture_io{
            v1.data(list: :zoom, format: :string, log: true).must_equal 'zoom!'
          }
          out.must_equal %Q{GET /cars/_design/cars/_list/zoom/v1?\n}
        end

        it 'show list iterating' do
          couch.with_fixtures! %w[make model],
                               %w[Audi A3   ],
                               %w[Audi A4   ]
          v1.keys.must_equal %w[A3 A4]
          couch.design_new :list, :iter , <<-HERE
(head, req) ->
  start
    "headers":
      "Content-Type": "text/html"
  loop
    return unless r = getRow()
    send r.key
HERE
          # v1.data(list: :iter, format: :raw).must_equal 'A3A4'
          out, err = capture_io{
            v1.data(list: :iter, format: :string, log: true, keys: ['A3']).must_equal 'A3'
          }
          out.must_equal %Q{POST /cars/_design/cars/_list/iter/v1 {:keys=>["A3"]}\n}

        end
      end
    end

    describe Couch::BulkImport do
      it do
        couch.blank!.bulk_import(2) do |bi|
          5.times{|i| bi.doc_new nr: i}
        end
        couch.db_info[:doc_count].must_equal 5
      end
    end

    describe "Documents" do

      it "#doc with no document" do
        couch.doc(:not_existing_id).must_be_nil
      end

      it "#doc_new create new id" do
        couch.doc_new(nil, name: 'frank')[:ok].must_equal true
      end

      it "#doc_new get update" do
        doc_n = couch.blank!.doc_new(:frank, name: 'frank')
        doc_n.keys.must_equal %w[ok id rev]

        doc_g = couch.doc(:frank)
        doc_g.keys.must_equal %w[_id _rev name]
        doc_n[:rev].must_equal doc_g[:_rev]

        doc_u = couch.doc_update(doc_g, age: 47)
        doc_u.keys.must_equal %w[ok id rev]
      end

      it "#doc_create_or_update" do
        couch.blank!.doc_create_or_update 'bla', name: 'frank'
        couch.doc('bla')[:name].must_equal 'frank'
        couch.doc_create_or_update 'bla', name: 'dirk'
        couch.doc('bla')[:name].must_equal 'dirk'
      end

      it '#doc_new_bulk' do
        couch.blank!.doc_new_bulk [{name: :Frank},
          {name: :Dieter}]
        couch.db_info[:doc_count].must_equal 2
      end

      it "#next_id" do
        (0..19).map{ couch.next_id }.uniq.size.must_equal 20
      end

      it "#doc_merge" do
        address = Map.new.set :address, :street, 'Seidenstrasse'
        couch_one.doc_merge :frank, address
        couch_one.doc(:frank)[:address][:street].must_equal 'Seidenstrasse'
      end

      it "#attach" do
        couch.blank!
        couch.doc_attach_app "d3", "test/fixtures/d3"
      end

      describe '#doc_merge_cached' do
        it "doc_merge_cached" do
          couch.blank!.doc_merge_cached :frank, adr: { street: 'seiden'}
          couch.doc_merge_cached        :frank, adr: { plz: 51063   }
          couch.doc(:frank)[:adr].keys.must_equal %w[street plz]
        end

        it "fetches document when its not on the cache" do
          couch.blank!.doc_new   :frank, adr: { street: 'seiden'}
          couch.doc_merge_cached :frank, adr: { plz: 51063   }
          couch.doc(:frank)[:adr].keys.must_equal %w[street plz]
        end
      end

      describe "show action" do
        it "show without document" do
          couch.blank!.design_new :show,:s1,'(doc, req) -> "hallo welt"'
          couch.show(:s1).must_equal 'hallo welt'
        end

        it "show document" do
          couch_one.design_new :show,:s2,'(doc, req) -> "hallo #{doc.name}"'
          couch.show(:s2,:frank).must_equal 'hallo Frank'
        end
      end
    end
  end
end
